// 
//  SCSI_Inspector.app - a SCSI reporting tool for Motorola and Intel 
//						 systems running NeXTSTEP version 3.2 or higher.
//
//  Copyright (C) 1994 by Christopher Wolf.
//  
//	This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  Bug-reports, comments and questions should be directed to:
//		Christopher Wolf - chris@alchemy.geo.cornell.edu
//
//  Portions of this program are based on earlier code (SCSI Inquirer)
//  Copyright 1992 by Jiro Nakamura and used with permission.
//

//
// 	For best results view this file with a tab size of 4 and
//	a display width of 132 columns or wider.
//

// 	file version information
#define RCSFM "$Id: Formatter.m,v 0.15 94/11/28 00:37:20 chris Exp $"

// 	NeXTSTEP headers
#import <appkit/appkit.h>

// 	generic Unix headers
#import <math.h>
//#import <strings.h>
//#import <libc.h>
//#import <stdio.h>


//	SCSI_Inspector.app headers
#import "SCSI2_Kit.subproj/SCSI.h"
#import "SCSI2_Kit.subproj/DriveSCSI.h"
#import "Formatter.h"
#import "Inspector.h"

// string constants
const char RCSfm[] 	= RCSFM;
const char RCSfh[] 	= RCSFH;	
	
@implementation Formatter


//	********************************
//
//	Delegate methods for FormatPanel
//
//	********************************
	
- windowDidBecomeKey: sender
	{
	
	DriveSCSI *scsi;
	int result = -1;
	mode_page *mp = malloc(255);

	if ((scsi = [[DriveSCSI alloc] init]) && (![scsi setTarget: [inspector trg] lun: [inspector lun]]))
		{
		if (!(result = [scsi modeSenseCooked: mp size: 200 page: FORMATDEVICEPAGE pc: 0]))				
			{
			[formatInterleaveTextField setIntValue: readBigEndianShort(&mp->fodp.fodp_interleave_BE16)];
			[formatSectorSizeTextField setIntValue: readBigEndianShort(&mp->fodp.fodp_databytespersector_BE16)];
			if (!(mp->fodp.fodp_hsec))
				{
				[formatSectorSizeTextField setEnabled: NO];
				}			
			}
		[scsi free];
		}

	if (result)
		{
		NXRunAlertPanel("Format Drive","Unable to determine interleave and block size settings for this drive!",
						"OK",NULL,NULL);
		}

	free(mp);
	return self;
	
	}



//	************************************
//
//	Instance methods for Formatter class
//
//	************************************

- formatTarget: sender
	{
	
	DriveSCSI *scsi;
	char msgstring[512];
	mode_parameter_list *mpl = malloc(255);
	mode_page *mp;
	int result = -1, interleave, blockSize;
	
	if ((interleave = [formatInterleaveTextField intValue]) > 16)
		{
		interleave = 16;
		}
	if ((blockSize = ([formatSectorSizeTextField intValue]/512)*512) < 512)
		{
		blockSize = 512;
		}

	[formatPanel close];
	
	sprintf(msgstring,
	 		"This format operation will erase all data on the %s %s hard disk located at SCSI target %d, "
			"logical unit number %d.  The current grown defect list will be %s.  Media certification "
			"procedures %s be performed.  An interleave value of %d and a block size of %d will be "
			"specified.  This operation may take a long time to complete on some devices.  Are you sure "
			"you want to proceed?",
			[inspector targetVendorID], [inspector targetProductID], [inspector trg], [inspector lun],
			[formatPreserveGListButton state] ? "preserved" : "cleared",
			[formatMediaCertButton state] ? "will" : "will not", interleave, blockSize);
					   
	if(!NXRunAlertPanel("Format Drive",msgstring,"Cancel", "Format", NULL))
		{
		if ((scsi = [[DriveSCSI alloc] init]) && (![scsi setTarget: [inspector trg] lun: [inspector lun]]))
		  	{
			if (![formatReturnImmedButton state])
				{
				[scsi setImmediate: FALSE];
				[scsi setTimeout: 3*60*60];
				}				
			if ( (![scsi modeSenseRaw: mpl size: 200 page: FORMATDEVICEPAGE pc: 0 dbd: TRUE]) )				
				{
				mp = (mode_page *)(((char *)mpl)+4);
				mpl->mpl_datalength = 0;
				mpl->mpl_dsp.dsp_dreserved2a = 0;
				mpl->mpl_dsp.dsp_dreserved2b = 0;
				mp->fodp.fodp_ph.ph_reserved1 = 0;
				mp->fodp.fodp_ph.ph_ps = 0;
				mp->fodp.fodp_reserved20 = 0;
				mp->fodp.fodp_reserved21 = 0;
				mp->fodp.fodp_reserved22 = 0;
				mp->fodp.fodp_reserved23 = 0;
				writeBigEndianShort(0,&mp->fodp.fodp_sectorspertrack_BE16);
				writeBigEndianShort(blockSize,&mp->fodp.fodp_databytespersector_BE16);
				if (![scsi modeSelectRaw: mpl size: (mp->fodp.fodp_ph.ph_pagelength)+6 save: TRUE])
					{ 
				 	result = [scsi formatUnitInterleave: interleave 
							 clearGList: ![formatPreserveGListButton state]
							 cert: [formatMediaCertButton state]]; 
					}
				}
			[scsi free];
			}
		if (result)
		   	{
			[inspector reInspectTarget: self];
			NXRunAlertPanel("Format Drive","Error formatting drive!","OK",NULL,NULL);
			}
		else
			{
			if ([formatReturnImmedButton state])
				{
				NXRunAlertPanel("Format Drive","Format started successfully.","OK",NULL,NULL);
				}
			else
				{
				[inspector reInspectTarget: self];
				NXRunAlertPanel("Format Drive","Format completed successfully.","OK",NULL,NULL);
				}
			}
		}

	free(mpl);
	return self;

	}
		
@end
