// 
//  SCSI_Inspector.app - a SCSI reporting tool for Motorola and Intel 
//						 systems running NeXTSTEP version 3.2 or higher.
//
//  Copyright (C) 1994 by Christopher Wolf.
//  
//	This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  Bug-reports, comments and questions should be directed to:
//		Christopher Wolf - chris@alchemy.geo.cornell.edu
//
//  Portions of this program are based on earlier code (SCSI Inquirer)
//  Copyright 1992 by Jiro Nakamura and used with permission.
//

//
// 	For best results view this file with a tab size of 4 and
//	a display width of 132 columns or wider.
//

// 	file version information
#define RCSIM "$Id: Inspector.m,v 0.226 94/12/07 02:02:35 chris Exp $"

// 	NeXTSTEP headers
#import <appkit/appkit.h>

// 	generic Unix headers
#import <strings.h>
#import <libc.h>
#import <stdio.h>
#import <stdlib.h>
#import <sys/types.h>
#import <sys/time.h>

//	SCSI_Inspector.app headers
#import "SCSI2_Kit.subproj/SCSI.h"
#import "SCSI2_Kit.subproj/DriveSCSI.h"
#import "Inspector.h"

// string constants
const char RCSim[] 	= RCSIM;
const char RCSih[] 	= RCSIH;
static char reserved[] 	= "Reserved";
static char pDep[] 		= "Peripheral dependent";
	

void removeLastSpace( char *s, int length)
	{
	for(length -= 1; length>0 && (s[length] ==' '||s[length]=='\0'); length--)
		{
		s[length] = '\0';
		}
	}
	

@implementation Inspector

- (int)setTarget: (int) trg lun: (int) ln
	{
	
	if ((target != trg) || (lun != lun))
		{
		target   = trg;
		lun		 = ln;
		lastMode = -2;
		}
	
	return 0;
	
	}
	
- (BOOL)inquireTarget
	{
	
	SCSI *scsi;	 
	
	devInfo[target].deviceValid = NO;

	scsi = [[SCSI alloc] init];
	if (![scsi setTarget: target lun: lun]) 
		{
		
		if ( [scsi isDevice] && ![scsi inquiry: &(devInfo[target].irBuffer)] )
			{
			devInfo[target].deviceValid = YES;
			strcpy(devInfo[target].typeString,[self deviceTypeDescription: devInfo[target].irBuffer.ir_devicetype]);
			removeLastSpace(devInfo[target].irBuffer.ir_vendorid, 8);
			removeLastSpace(devInfo[target].irBuffer.ir_productid, 16);
			}

		}
	[scsi free];
	
	return devInfo[target].deviceValid;	
	
	}	
	
- reInspectTarget: sender
	{
	lastMode = -2;
	return [self inspectTarget: sender];
	}

- inspectTarget: sender
	{
	
	char tempBuffer[64];

	sprintf(tempBuffer, "SCSI Target #: %d", target);
			
	if (![self targetIsValid])
		{
		
		[scsi2Text setText: ""];
		[deviceNameText setStringValue: "No Device"];
		[deviceNumText setStringValue: tempBuffer];
		[popupButton setEnabled: FALSE];		

		}
	else
		{

		[deviceNumText setStringValue: tempBuffer];
		sprintf(tempBuffer, "%.8s %.16s", [self targetVendorID], [self targetProductID]);
		[deviceNameText setStringValue: tempBuffer];	
	
		if (   ([self targetDeviceType] == DEVTYPE_DISK)
			|| ([self targetDeviceType] == DEVTYPE_CDROM)
			|| ([self targetDeviceType] == DEVTYPE_WORM)
			|| ([self targetDeviceType] == DEVTYPE_OPTICAL))
			{
			[benchmarksItem setEnabled: TRUE];
			}
		else
			{
			[benchmarksItem setEnabled: FALSE];
			}

		popupList = [popupButton target];
		[[popupList itemList] selectCellAt: 0: 0];
		[popupButton setTitle: [popupList selectedItem]];
		[popupButton setEnabled: TRUE];	

		[self outputModeInfo: self];

		}
	
	return self;

	}

- chooseMode: sender
	{

	return [self perform:(@selector(outputModeInfo:)) with:self afterDelay:1 cancelPrevious:YES];
	
	}

- outputModeInfo: sender
	{
	
	int outputMode;
	DriveSCSI *scsi;
	NXPoint origin = {0.0, 0.0};	

	if ((outputMode = [popupList indexOfItem: [popupList selectedItem]]) != lastMode)
		{
		
		if (output = NXOpenMemory(NULL, 0, NX_READWRITE)) 
			{
			
			[scsi2Text setText: ""];
			[inspectorWindow display];
			
			NXPrintf(output, "{\\rtf0\\ansi{\\fonttbl\\f0\\fnil %s;\\f1\\fmodern %s;}\n"
					"\\pard\\tx400\\tx3850\\fi-3850\\li3850\\f0\\b0\\i0\\ul0\\fs%d\n",	
					"Helvetica", "Courier", (int) 12 * 2);

			scsi = [[DriveSCSI alloc] init];
			[scsi openTarget: target lun: lun];
					
			switch (outputMode)
				{
				case 2:
					{
					if( NXRunAlertPanel("Perform Benchmark",
						"To ensure accurate results this benchmark must be run "
						"when there is minimal system load and when no disk activity is "
						"occurring.  This benchmark may take several minutes to complete.", 
						"Start", "Cancel", NULL))
						{
						[self outputBenchmarks: scsi];
						}
					else
						{
						outputMode = -1;
						NXPrintf(output,"Benchmark cancelled!");
						}
					break;
					}
				case 1:
					{
					if( NXRunAlertPanel("Perform Diagnostics",
						"To ensure accurate results diagnostics should only be "
						"performed when no other disk activity is occurring.  "
						"Diagnostics may take 20 or 30 seconds to complete.", 
						"Start", "Cancel", NULL))
						{
						[self outputDiagnostics: scsi];
						}
					else
						{
						outputMode = -1;
						NXPrintf(output,"Diagnostics cancelled!");
						}
					break;
					}
				default:
					{
					outputMode = 0;
					}
				case 0:
					{
					[self outputInfo: scsi];
					break;
					}
				}
			
			[scsi closeSCSI];
			[scsi free];
		
			[inspectorWindow disableDisplay];
			NXSeek(output, 0L, NX_FROMSTART);
			[scsi2Text readRichText: output];
			[scsi2Text scrollPoint: &origin];
			[inspectorWindow display];
		
			lastMode = outputMode;
	
			NXCloseMemory(output, NX_FREEBUFFER);
			output = NULL;

			}
		else
			{

			fprintf(stderr,"SCSI Inspector: Couldn't open memory stream for output, fatal error. Quitting.");
			exit(1);

			}
		
		}
			
	return self;

	}

	
//
// Output general device information 
//

- outputInfo: (DriveSCSI *)scsi
	{
	
	struct capacity_reply cr;
	int numBlocks,blklen;
	mode_parameter_list *mpl = malloc(256);
	mode_page *mp;

	NXPrintf(output, "\\b\\ul Inquiry Command\\b0\\ul0\\\n\\\n"
					 "\tSCSI Target #:\t%d\\\n"
					 "\tDevice Type:\t%s\\\n"
					 "\tVendor ID:\t%.8s\\\n"
					 "\tProduct ID:\t%.16s\\\n"
					 "\tRevision:\t%.4s\\\n"
					 "\tANSI Standard:\t%s\\\n"
					 "\tRelative Addressing:\t%s\\\n"
					 "\t32 Bit Wide Transfers:\t%s\\\n"
					 "\t16 Bit Wide Transfers:\t%s\\\n"
					 "\tSynchronous Transfers:\t%s\\\n"
					 "\tLinked Commands:\t%s\\\n"
					 "\tTagged Command Queuing:\t%s\\\n"
					 "\tSoft Reset:\t%s\\\n\\\n",
					 target,
					 [self targetDeviceTypeString],
					 [self targetVendorID],
					 [self targetProductID],
					 devInfo[target].irBuffer.ir_revision,
					 [self ansiVersionDescription: devInfo[target].irBuffer.ir_ansiversion],
					 devInfo[target].irBuffer.ir_reladr ? "Yes" : "No",
					 devInfo[target].irBuffer.ir_wbus32 ? "Yes" : "No",
					 devInfo[target].irBuffer.ir_wbus16 ? "Yes" : "No",
					 devInfo[target].irBuffer.ir_sync   ? "Yes" : "No",
					 devInfo[target].irBuffer.ir_linked ? "Yes" : "No",
					 devInfo[target].irBuffer.ir_cmdque ? "Yes" : "No",
					 devInfo[target].irBuffer.ir_sftre  ? "Yes" : "No");

	if(![scsi readCapacity: &cr])
		{
		numBlocks = readBigEndianLong(&cr.cr_lastlba_BE32)+1;
		blklen    = readBigEndianLong(&cr.cr_blklen_BE32);
		NXPrintf(output,
				"\\b\\ul Read Capacity Command\\b0\\ul0\\\n\\\n"
				"\tNumber of Blocks:\t%d\\\n"
				"\tBlock Size:\t%d\\\n"
				"\tDisc Capacity:\t%d K   (%d MB)\\\n\\\n",
				numBlocks,
			    blklen,
			    numBlocks * blklen / (1024),									
				numBlocks * blklen / (1024*1024));
		}

	switch([self targetDeviceType])
		{
		case DEVTYPE_TAPE:
			{	 		
			if (![scsi modeSenseRaw: mpl size: 200 page: DEVICECONFIGPAGE pc: 0 dbd: FALSE])
				{
				mp = (mode_page *)(((char *)mpl)+4+mpl->mpl_bdlength);
				NXPrintf(output, 
						"\\b\\ul Mode Parameter Header\\b0\\ul0\\\n\\\n"
						"\tDensity Code:\t%s  (%02Xh)\\\n"
						"\tSpeed:\t%s  (%02Xh)\\\n"
						"\tBuffered Mode:\t%s\\\n"
						"\tWrite Protected:\t%s\\\n\\\n"
						"\\b\\ul Device Configuration Page\\ul0\\b0\\\n\\\n"			
						"\tChange Active Partition:\t%s\\\n"			
						"\tChange Active Format\t%s\\\n"			
						"\tActive Partition:\t%02Xh\\\n"
						"\tActive Format:\t%02Xh\\\n"			
						"\tWrite Buffer Full Ratio:\t%d\\\n"			
						"\tRead Buffer Empty Ratio:\t%d\\\n"			
						"\tWrite Delay Time:\t%d ms\\\n"			
						"\tData Buffer Recovery:\t%s\\\n"
						"\tBlock Indentifiers:\t%s\\\n"
						"\tSetmarks:\t%s\\\n"
						"\tVelocity Control:\t%s\\\n"
						"\tStop On Consecutive Marks:\t%d\\\n"
						"\tRecover Buffer Order:\t%s\\\n"
						"\tReport Early Warning:\t%s\\\n"
						"\tGap Size:\t%02Xh\\\n"
						"\tEnd of Data Definition:\t%s\\\n"
						"\tEnd of Data Generation:\t%s\\\n"
						"\tSynchronize at Early Warning:\t%s\\\n"
						"\tBuffer size at Early Warning:\t%04Xh\\\n"
						"\tData Compression Algorithm:\t%s  (%02Xh)\\\n\\\n",
						[self densityCodeDescription: mpl->mpl_bd[0].bd_densitycode], mpl->mpl_bd[0].bd_densitycode,
						[self speedDescription: mpl->mpl_dsp.dsp_sspeed],mpl->mpl_dsp.dsp_sspeed,
						[self bufModeDescription: mpl->mpl_dsp.dsp_sbufferedmode], 
						mpl->mpl_dsp.dsp_swp ? "Yes" : "No",
						mp->dcp.dcp_cap ? "Yes" : "No",
						mp->dcp.dcp_caf ? "Yes" : "No",
						mp->dcp.dcp_activepartition,
						mp->dcp.dcp_activeformat,
						mp->dcp.dcp_wbfr,
						mp->dcp.dcp_rber,
						readBigEndianShort(&mp->dcp.dcp_writedelay_BE16) * 100,
						mp->dcp.dcp_dbr ? "Supported" : "Not supported",		
						mp->dcp.dcp_bis ? "Supported" : "Not supported",
						mp->dcp.dcp_rsmk ? "Recognize/Reported" : "Not supported",
						mp->dcp.dcp_avc ? "Automatic" : "Default",
						mp->dcp.dcp_socf,
						mp->dcp.dcp_dbr ? (mp->dcp.dcp_rbo ? "LIFO" : "FIFO") : "Not applicable",
						mp->dcp.dcp_rew ? "Reported for READs" : "Not reported for READs",
						mp->dcp.dcp_gapsize, 
						[self eodDescription: mp->dcp.dcp_eod],
						mp->dcp.dcp_eeg ? "Enabled" : "Disabled",
						mp->dcp.dcp_sew ? "Synchronize" : "Retain",
						readBigEndianValue(&mp->dcp.dcp_earlywarning_BE24, 24),
						[self compressionDescription: mp->dcp.dcp_sdca], 
						mp->dcp.dcp_sdca);
				}
			break;
			}
		case DEVTYPE_CDROM:
			{
			if (![scsi modeSenseRaw: mpl size: 200 page: CDROMPARAMETERSPAGE pc: 0 dbd: FALSE])
				{
				NXPrintf(output, 
						"\\b\\ul Mode Parameter Header\\b0\\ul0\\\n\\\n"
						"\tMedium Type:\t%s  (%02Xh)\\\n"
						"\tDensity Code:\t%s  (%02Xh)\\\n\\\n",
						[self mediumTypeDescription: mpl->mpl_mediumtype], mpl->mpl_mediumtype,
						[self densityCodeDescription: mpl->mpl_bd[0].bd_densitycode],
						mpl->mpl_bd[0].bd_densitycode);
				}
			break;
			}
		case DEVTYPE_DISK:
			{
			if(![scsi modeSenseRaw: mpl size: 200 page: RIGIDDISKPAGE pc: 0 dbd: FALSE])
				{
				mp = (mode_page *)(((char *)mpl)+4+mpl->mpl_bdlength);
				NXPrintf(output, 
						"\\b\\ul Mode Parameter Header\\b0\\ul0\\\n\\\n"
						"\tMedium Type:\t%s  (%02Xh)\\\n"
						"\tWrite Protected:\t%s\\\n\\\n"
						"\\b\\ul Rigid Disk Geometry Page\\b0\\ul0\\\n\\\n" 
						"\tNumber of Heads:\t%d\\\n"
						"\tNumber of Cylinders:\t%d\\\n"			
						"\tMedium Rotation Rate:\t%d rpm\\\n\\\n",
						[self mediumTypeDescription: mpl->mpl_mediumtype], mpl->mpl_mediumtype,
						mpl->mpl_dsp.dsp_dwp  ? "Yes" : "No",
						mp->rdp.rdp_numheads,
						readBigEndianValue(&mp->rdp.rdp_numcyls_BE24, 24),
						readBigEndianShort(&mp->rdp.rdp_rotrate_BE16) );	
				}
			break;
			}
		}
	
	mp = (mode_page *)mpl;
	if (![scsi modeSenseCooked: mp size: 200 page: FORMATDEVICEPAGE pc: 0])
		{
		NXPrintf(output, 
				"\\b\\ul Format Device Page\\b0\\ul0\\\n\\\n"
				"\tSectors per Track:\t%d\\\n"			
				"\tBytes per Physical Sector:\t%d\\\n"			
				"\tInterleave:\t%d\\\n"
				"\tTracks per Zone:\t%d\\\n"			
				"\tAlt. Sectors per Zone:\t%d\\\n"			
				"\tAlt. Tracks per Zone:\t%d\\\n"			
				"\tAlt. Tracks per Logical Unit:\t%d\\\n"			
				"\tTrack Skew Factor:\t%d\\\n"
				"\tCylinder Skew Factor:\t%d\\\n"			
				"\tSoft Sector Formatting:\t%s\\\n"
				"\tHard Sector Formatting:\t%s\\\n"
				"\tRemovable Media:\t%s\\\n\\\n",
				readBigEndianShort(&mp->fodp.fodp_sectorspertrack_BE16),
				readBigEndianShort(&mp->fodp.fodp_databytespersector_BE16),
				readBigEndianShort(&mp->fodp.fodp_interleave_BE16),
				readBigEndianShort(&mp->fodp.fodp_tracksperzone_BE16),
				readBigEndianShort(&mp->fodp.fodp_altsectorsperzone_BE16),
				readBigEndianShort(&mp->fodp.fodp_alttracksperzone_BE16),
				readBigEndianShort(&mp->fodp.fodp_alttracksperunit_BE16),
				readBigEndianShort(&mp->fodp.fodp_trackskewfactor_BE16),
				readBigEndianShort(&mp->fodp.fodp_cylinderskewfactor_BE16),
				mp->fodp.fodp_ssec ? "Supported" : "Not supported",
				mp->fodp.fodp_hsec ? "Supported" : "Not supported",
				mp->fodp.fodp_rmb ? "Supported" : "Not supported");		
		}

	if (![scsi modeSenseCooked: mp size: 200 page: CACHINGPAGE pc: 0])
		{
		NXPrintf(output, 
				"\\b\\ul Cache Page\\b0\\ul0\\\n\\\n" 
				"\tWrite Cache Enabled:\t%s\\\n"
				"\tRead Cache Enabled:\t%s\\\n"
				"\tDisable Prefetch Limit:\t%d\\\n"
				"\tPrefetch Ceiling (blocks):\t%d\\\n",
				mp->cp.cp_wce ? "Yes" : "No",
				mp->cp.cp_rcd ? "No" : "Yes",
				readBigEndianShort(&mp->cp.cp_dptl_BE16),
				readBigEndianShort(&mp->cp.cp_maxprefetchceiling_BE16));
		if (mp->cp.cp_mf)
			{
			NXPrintf(output,
					"\tMin. Prefetch (multiplier):\t%d\\\n"			
					"\tMax. Prefetch (multiplier):\t%d\\\n\\\n",
					readBigEndianShort(&mp->cp.cp_minprefetch_BE16),
					readBigEndianShort(&mp->cp.cp_maxprefetch_BE16));
			}
		else
			{
			NXPrintf(output,
					"\tMin. Prefetch (blocks):\t%d\\\n"
					"\tMax. Prefetch (blocks):\t%d\\\n\\\n",
					readBigEndianShort(&mp->cp.cp_minprefetch_BE16),
					readBigEndianShort(&mp->cp.cp_maxprefetch_BE16));
			}
		}
		
	free(mpl);				
	return self;
	
	}


- outputDiagnostics: (DriveSCSI *)scsi
	{
			
	NXPrintf(output,
			"\\b\\ul Generic Diagnostics\\b0\\ul0\\\n\\\n"	
			"\tTest Unit Ready:\t");
			
	if(![scsi testUnitReady])
		{ 
		NXPrintf(output, "Pass\\\n");
		}
	else
		{
		NXPrintf(output, "Fail  (Unit not Ready)\\\n");
		}
		
	NXPrintf(output,"\tSelf Check:\t");
	if(![scsi sendDiagnostic: NULL size: 0 pf: NO st: YES dol: NO uol: NO])
		{
		NXPrintf(output, "Pass\\\n");
		}
	else
		{
		NXPrintf(output, "Fail\\\n");
		}
		
	return self;

	}


- outputBenchmarks: (DriveSCSI *)scsi
	{
	 
	int iterations = 64, bufSizes[7] = {512,1024,2*1024,8*1024,32*1024,128*1024,0};
		
	NXPrintf(output, "\\b\\ul SCSI Inspector Benchmark v2 Results\\b0\\ul0\\\n\\\n"
			"\tVendor ID:\t%.8s\\\n"
			"\tProduct ID:\t%.16s\\\n\\\n",
			[self targetVendorID],
			[self targetProductID]);
			
	NXPrintf(output, "\\b\\ul Read from disk\\b0\\ul0\\\n\\\n");
	[self doBenchDrive: scsi Sector: -1 BufSizes: bufSizes Iterations: iterations];
				
	NXPrintf(output, "\\b\\ul Read from cache\\b0\\ul0\\\n\\\n");		
	[self doBenchDrive: scsi Sector: 0 BufSizes: bufSizes Iterations: iterations];

	return self;
	
	}
	
- doBenchDrive: (DriveSCSI *)scsi Sector: (int)Sector BufSizes: (int *)bufSizes Iterations: (int)iterations
	{

	int count, secArray[iterations], bufSize, bufSizeIndex, numBlocks, blkSize, result = 0;
	float execTime;
	char *dest;
	struct capacity_reply cr;
	struct scsi_req *sr;
	
	// get block size and block number information
	[scsi readCapacity: &cr];
    numBlocks = readBigEndianLong(&cr.cr_lastlba_BE32)-1024;
	blkSize   = readBigEndianLong(&cr.cr_blklen_BE32);
	
	// loop through all specified buffer sizes
	for (bufSizeIndex = 0; (bufSize = bufSizes[bufSizeIndex]); bufSizeIndex++)
		{

		if (   (bufSize < blkSize) 
			|| ( (devInfo[target].irBuffer.ir_ansiversion < 2) && (bufSize/blkSize>256) ) )
			{
			continue;
			}
			
		dest = malloc(bufSize);
		
		// select sector start values (random if Sector == -2, last block if Sector == -1, otherwise == sector)
		for (count = 0; count < iterations; count++)
			{
			switch (Sector)
				{
				case -1:
					{
					secArray[count] = (int)(((float)(random()&0xFFFF)/65535.0)*numBlocks);
					break;
					}
				default:
					{
					secArray[count] = Sector;
					break;
					}
				}
			}
			
		// perform timed benchmark
		sr = [scsi statusReq];
		execTime = 0.0;				
		for (count = 0; count < iterations; count++)
			{
			if (devInfo[target].irBuffer.ir_ansiversion < 2)
				{
				result = [scsi read: dest numBytes: bufSize numBlocks: bufSize/blkSize lba: secArray[count]];
				}
			else
				{
				result = [scsi readExtended: dest numBytes: bufSize numBlocks: bufSize/blkSize lba: secArray[count]];
				}
			execTime += ((sr->sr_exec_time.tv_sec)*1000000.0 + (sr->sr_exec_time.tv_usec));	
			}
		
		// calculate and output results
		if (!result)
			{
			NXPrintf(output, "\t%d Byte Buffer\t%.3f MB/Sec\\\n", bufSize, (float)bufSize*iterations/execTime);
			}
			
		free(dest);
		
		}
		
	NXPrintf(output, "\\\n");
		
	return self;
					
	}	

	
// 
// Return string describing specified device type
//

- (const char *)deviceTypeDescription: (int) deviceType
	{

	switch(deviceType)
		{
		case DEVTYPE_DISK:
			{
			return("Hard-Drive");
			}
		case DEVTYPE_TAPE:														
			{
			return("Tape-Drive");
			}
		case DEVTYPE_PRINTER:
			{
			return("Printer");
			}
		case DEVTYPE_PROCESSOR:
			{
			return("Processor");
			}
		case DEVTYPE_WORM:
			{
			return("Worm-Drive");
			}
		case DEVTYPE_CDROM:
			{
			return("CD-ROM");
			}
		case DEVTYPE_SCANNER:
			{
			return("Scanner");
			}
		case DEVTYPE_OPTICAL:
			{
			return("Optical");
			}
		case DEVTYPE_NOTPRESENT:
			{
			return("No-Device");
			}
		default:
			{
			return("Not-Recognized");
			}
			
		}				
	}


//
// Return string describing specified ANSI version
//

- (const char *)ansiVersionDescription: (int)ansiVersion
	{
	
	static char *ansiVersionStrings[8] =
		{
		"Not compliant", "X.131-1986 (SCSI-1)", "X.131-198X (SCSI-2)",
		reserved, reserved, reserved, reserved, reserved
		};

	return ansiVersionStrings[ansiVersion];
	
	}		


//
// Return string describing specified tape buffer mode
//

- (const char *)bufModeDescription: (int)bufMode
	{

	static char *bufModeString[8] = 
		{ 
		"Target will not report GOOD until data blocks are actually written",
		"Target will report GOOD as soon as all blocks to be written are in the buffer",
		"Target will report GOOD as soon as all previously buffered blocks have been"
			"written and the data from the current write command has been received",
		 reserved, reserved, reserved, reserved, reserved,
		 };		 
		 
	return bufModeString[bufMode];
	
	}


//
// Return string describing specified tape speed
//

- (const char *)speedDescription: (int)speedCode
	{
	
	static char *speedString[16] = 
		{
		"Default speed", "Slowest speed", pDep, pDep, pDep, pDep, pDep, pDep,
		pDep, pDep, pDep, pDep, pDep, pDep, pDep, pDep,
		};
	
	return speedString[speedCode];
	
	}	


//
// Return string describing tape EOD
//

- (const char *)eodDescription: (int)eod
	{
		
	static char * eodDefinition[8] = 
		{
		"Logical unit's default",
		"Format-defined erased area",
		"As specified in SOCF field",
		"EOD recognition/generation not supported",
		reserved, reserved, reserved, reserved,
		};
		
	return eodDefinition[eod];
	}
	

//
// Return string describing tape compression
//

- (const char *)compressionDescription: (int)sdca
	{
	
	static char * compression[4] =  
		{ 
		"No compression", 
		"Default compression enabled", 
		reserved, 
		"Vendor specific"
		};		

	if( sdca > 0x01 && sdca < 0x80)
		{
		sdca = 0x02;
		}
		
	if( sdca > 0x7f )
		{
		sdca = 0x03;		
		}
		
	return compression[sdca];
	
	}

		
//
// Return string describing specified density code.
//

- (const char *)densityCodeDescription: (int)densityCode
	{

	static char *tapeDensityCodeString[24] = 
		{
		reserved, 
		"ANSI X3.22-1983 -- Width 0.5\", 9 tracks, 800 bpi, "
			"Non Return to Zero/change on ones recording, "
			"Reel-to-reel media, Parallel recorded",  
		"ANSI X3.39-1986 -- Width 0.5\", 9 tracks, 1600 bpi, "
			"Phase-encoded, Reel-to-reel media, Parallel recorded", // 0x02
		"ANSI X3.54-1986 -- Width 0.5\", 9 tracks, 6250 bpi, "
			"Group code recording, Reel-to-reel media, Parallel recorded",  //0x03
		"ANSI X.3.136-198 / QIC-11 -- Width 0.25\", 4/9 tracks, "
			"8000 bpi, Group code recording, Cartridge Media, Serial recorded", // 0x04
		"ANSI X.136-1986 -- Width 0.25\", 4/9 tracks, "
			"8000 bpi, Group code recording, Cartridge Media, Serial recorded", // 0x05
		"ANSI X3.157-1987 -- Width 0.5\", 9 tracks, 3200 bpi, "
			"Phase-encoded, Reel-to-reel media, Parallel recorded", // 0x06
		"ANSI X3.116-1986 -- Width 0.25\", 4 tracks, 6400 bpi, "
			"Inverted Modified Frequency Modulation encoding, "
			"Cartridge media, Serial recorded", // 0x07
		"ANSI X3.158-1987 -- Width 0.15\", 4 tracks, 8000 bpi, "
			"Group code recording, Cassette media, Serial recorded", // 0x08
		"ANSI X3B5/87-099 -- Width 0.5\", 18 tracks, 38,871 bpi, "
			"Group code recording, Cartridge media, Parallel recorded", // 0x09
		"ANSI X3B5/87-199 -- Width 0.5\", 22 tracks, 6667 bpi, "
			"Modified Frequency Modulation encoded "
			"Cartridge media, Serial recorded", // 0x0A
		"ANSI X3.56-1986 -- Width 0.25\", 4 tracks, 1600 bpi, "
			"Phase encoded, Cartridge media, Serial recorded", // 0x0B
		"HI-TC1 -- Width 0.5\", 24 tracks, 12690 bpi, "
			"Group code recording, Cartridge media, Serial recorded", // 0x0C
		"HI-TC2 -- Width 0.5\", 24 tracks, 25380 bpi, "
			"Group code recording, Cartridge media, Serial recorded", // 0x0D
		"Reserved for ECMA", // 0x0E
		"QIC120 -- Width 0.25\", 15 tracks, 10000 bpi, "
			"Group code recording, Cartridge media, Serial recorded", // 0x0F
		"QIC150 -- Width 0.25\", 18 tracks, 10000 bpi, "
			"Group code recording, Cartridge media, Serial recorded", // 0x10
		"QIC320 -- Width 0.25\", 26 tracks, 16000 bpi, "
			"Group code recording, Cartridge media, Serial recorded", // 0x11
		"QIC600 -- Width 0.25\", 24 tracks, 36000 bpi, "
			"Non Return to Zero/change on ones recording, "
			"Cartridge media, Serial recorded", // 0x12
		"X3B5/88-185A (DAT)-- Width 3.81mm, 1 track, 61000 bpi, "
			"DAT Data Storage format, Cassette media, Helican scan recording", // 0x13
		"ANSI X3B5/88-185A (8mm)-- Width 8mm, 1 track, 54000 bpi, "
			"Cassette media, Helican scan recording", // 0x14
		reserved, // 0x15
		"No change from previous density", // 0x16
		"Vendor unique", // 0x17
		};		
	
	static char *cdDensityCodeString[] =
		{
		"Default density code",
        "User Data Only - 2048 Bytes per physical sector",
		"User Data plus Auxiliary Data field - 2336 bytes per sector",
		"4 Byte Tag Field, User Data plus Auxiliary Data - 2340 bytes per sector",
		"Audio Information Only  - (1/75th of a second per logical block)", 
        reserved,
		"Vendor specific"
		};

	switch ([self targetDeviceType])
		{
		
		case DEVTYPE_TAPE:
			{	
			if ( (densityCode > 0x15) && (densityCode < 0x7f) )
				{
				densityCode = 0x15;
				}
			if (densityCode == 0x7f)
				{
				densityCode = 0x16;
				}
			if (densityCode >= 0x80)
				{
				densityCode = 0x17;
				}
			return tapeDensityCodeString[densityCode];
			}
			
		case DEVTYPE_CDROM:
			{
			if ((densityCode >= 0x05) && (densityCode <= 0x7f))
				{
				densityCode = 5;
				}
			if (densityCode >= 0x80)
				{
				densityCode = 6;
				}
			return cdDensityCodeString[densityCode];
			}

		default:
			{
			return reserved;
			}
					
		}
	}
		

//
// Return string describing specified media type
//

- (const char *) mediumTypeDescription: (int) mt
	{
	
	const char *cdStrings[] =
		{
		"Default (only one type supported)",
		"120 mm CD-ROM data only",
        "120 mm CD-DA audio only",
        "120 mm CD-ROM data and audio combined",
		reserved,
		"80 mm CD-ROM data only",
        "80 mm CD-DA audio only",
        "80 mm CD-ROM data and audio combined",
        "Vendor unique"
		};

	switch ([self targetDeviceType])
		{

		case DEVTYPE_DISK:
			{
			switch( mt )
				{
				case 0x00:
					return "Default medium type";
				case 0x01:
					return "Flexible disk; SS; unspecified medium";
				case 0x02:
					return "Flexible disk; DS; unspecified medium";
				case 0x05:
					return "ANSI X3.73-1980 (8\" flexible disk)";
				case 0x06:
					return "See Appendix F for standard (8\" flexible disk)";
				case 0x09:
					return "No standard (8\" flexible disk)";
				case 0x0A:
					return "ANSI X3.121-1984 (8\" flexible disk)";
				case 0x0D:
					return "ANSI X3.82-1980 (5.25\" flexible disk)";
				case 0x12:
					return "ANSI X3.125-1985 (5.25\" flexible disk)";
				case 0x16:
					return "ANSI X3.126-1986 (5.25\" flexible disk)";
				case 0x1a:
					return "ISO DIS8630-1985 (5.25\" flexible disk)";
				case 0x1e:
					return "ANSI X3.137 (3.5\" flexible disk)";
				case 0x40:
					return "1/4\" magnetic tape (12 track)";
				case 0x44:
					return "1/4\" magnetic tape (24 track)";
				default:
					return "Unrecognized (no standard)";
				}
			}

		case DEVTYPE_CDROM:
			{
			if ((mt >= 0x08) && (mt <= 0x7f))
				{
				mt = 4;
				}
			if (mt >= 0x80)
				{
				mt = 8;
				}
			return cdStrings[mt];	
			}
			
		default:
			{
			return reserved;
			}

		}
	}
	

- (int) trg
	{
	return target;
	}
	
- (int) lun
	{
	return lun;
	}
	
//
// Is there a valid device at this ID?
//

- (BOOL) targetIsValid
	{
	return devInfo[target].deviceValid;
	}


//
// Is the device at this ID removable?
//

- (BOOL) targetIsRemovable
	{ 
	return devInfo[target].irBuffer.ir_removable; 
	}


//
// What type of device is at this ID? (returns numeric identifier)
//

- (int) targetDeviceType
	{ 
	return devInfo[target].irBuffer.ir_devicetype; 
	}
	
	
//
// What type of device is at this ID? (returns string identifier)
//

- (char *) targetDeviceTypeString
	{
	return devInfo[target].typeString;
	}


//
// What's the VendorID for this ID?
//

- (char *) targetVendorID
	{ 
	return devInfo[target].irBuffer.ir_vendorid; 
	}


// 
// What's the product ID for this ID?
//

- (char *) targetProductID
	{ 
	return devInfo[target].irBuffer.ir_productid; 
	}
	
@end
